#!/usr/bin/env python
#====================================================
# GoZ Script: Saving GoZ Mesh
# This file is for GoZ file-exchange with ZBrush
#====================================================
import lx;
import os;
import time;


gGoZBrushPath = "/Users/Shared/Pixologic/GoZBrush/"
gGoZAppExt = ".app"
gDebug = 0


# ===============
#  Debug purpose 
# ===============
def printDebug(str):
	if (gDebug):
		lx.out(str)
	return

def safeGetItemName():
	try:
		return lx.eval('item.name ?')
	except:
		return ""

def launchApp(exePath):
	# For Windows:
	#os.system(exePath)

	# For MacOSX:
	os.system('open %s' %(exePath))
	return

def createGoZBrushID(name):
	printDebug('CreateGoZBrushID(%s)' %(name))
	fileName = gGoZBrushPath + "GoZ_ObjectPath.txt"
	exePath = gGoZBrushPath + "GoZMakeObjectPath" + gGoZAppExt
	filesToDelete = []
	cont = 1
	while (cont == 1):
		file = open(fileName, "w")
		file.write(name)
		#file.write('%s' %(name))
		file.close()

		printDebug('    [Creating GoZBrushID for "%s"...' %(name))
		launchApp(exePath)
		time.sleep(0.1);
		maxTries = 50;
		uniquePath = name
		while (maxTries > 0):
			lineToPrint = ""
			try:
				lineToPrint = "open(" + fileName + ");"
				file2 = open(fileName, "r")
				lineToPrint = lineToPrint + "  seek(0);"
				file2.seek(0)
				lineToPrint = lineToPrint + "  readline();"
				uniquePath = file2.readline()
				lineToPrint = lineToPrint + "  [lineRead = " + uniquePath + "];  close();"
				file2.close()
				lineToPrint = lineToPrint + "  closed!!!"
				if (len(uniquePath) > len(name)):
					printDebug('     found GoZBrushID path "%s"]' %(uniquePath))
					break
				if (len(uniquePath) > 0):
					printDebug('     still searching GoZBrushID path "%s"...' %(uniquePath))
				else:
					printDebug('     still searching GoZBrushID, file empty...')
				time.sleep(0.1)
			except:
				lineToPrint = lineToPrint + "  error!!!"
				time.sleep(0.1)
			maxTries = maxTries-1
			if (lineToPrint != ""):
				printDebug('Attempt to open file: %s' %(lineToPrint))

		if (maxTries == 0):
			printDebug('     did not found any GoZBrushID]')

		# If another item has the same name as GoZID, searches for another GoZID.
		gozID = os.path.basename(uniquePath)
		if (gozID == name):
			break
		lx.eval('select.drop item')
		lx.eval('select.item "%s" set' %(gozID))
		#selName = lx.eval('item.name ?')
		selName = safeGetItemName()
		if (selName != gozID):
			break

		uniquePath = uniquePath + ".ztn"
		filesToDelete.append(uniquePath)
		file = open(uniquePath, "w")
		file.close()

	count = len(filesToDelete)
	for i in range(0, count):
		uniquePath = filesToDelete[i]
		os.remove(uniquePath)
	return gozID

def dialog(title, msg, type):
	try:
		lx.eval('dialog.setup %s' %(type))
		lx.eval('dialog.title "%s"' %(title))
		lx.eval('dialog.msg "%s"' %(msg))
		lx.eval('dialog.result ok')
		lx.eval('dialog.open')
		result = lx.eval('dialog.result ?')
		return result
		
	except:
		result = lx.eval('dialog.result ?')
		return result


def getSelMeshes():
	list1 = []
	list2 = []
	meshList = []
	sel = lx.eval('query sceneservice selection ? all')

	if type(sel) == tuple:
		count = len(sel)
		printDebug('%s objects selected' %(count))
		for i in range(0, count):
			itemID = sel[i]
			printDebug('sel #%s is "%s"' %(i, itemID))
			try:
  				itemType = lx.eval('!!query sceneservice item.type ? "%s"' %(itemID))
  				if itemType == 'groupLocator':
	  				list1.append(itemID)
		  		else:
			  		if itemType == 'mesh':
				  		meshList.append(itemID)
			except:
				printDebug('selected item #%s("%s") has unknown type' %(i, itemID))
	else:
		printDebug('sel = "%s"' %(sel))
		if type(sel) == str:
			printDebug('sel is not "None"')
			itemID = sel
			itemType = lx.eval('query sceneservice item.type ? %s' %(itemID))
			if itemType == 'groupLocator':
				list1.append(itemID)
			else:
				if itemType == 'mesh':
					meshList.append(itemID)
		else:
			printDebug('sel is "None"')

	printDebug('empty list ...')
	count = len(list1)
	printDebug('  ... count=%d' %(count))
	while count > 0:
		for i in range(0, count):
			itemID = list1[i]
			children = lx.eval('query sceneservice item.children ? %s' %(itemID))
			if type(children) == tuple:
				childrenList = children + ("unused1", "unused2")
			else:
				childrenList = (children, "unused1", "unused2")
			childrenCount = len(childrenList)-2
			for j in range(0, childrenCount):
				childID = childrenList[j]
				childType = lx.eval('query sceneservice item.type ? %s' %(childID))
				if childType == 'groupLocator':
					list2.append(childID)
				else:
					if childType == 'mesh':
						meshList.append(childID)
		list1 = list2
		list2 = []
		count = len(list1)
	return meshList


def doGoZExport():
	exportedCount = 43

	# Gets the GoZ project path.
	fileName = gGoZBrushPath + "GoZ_ProjectPath.txt"
	try:
		file = open(fileName, "r")
	except:
		printDebug('Error opening project path file!!!')

	try:
		projectPath = file.readline()
		file.close()
	except:
		printDebug('Error reading project path file!!!')
		file.close()

	printDebug('Before get sel meshes')
	exportedCount = 0
	selList = getSelMeshes()
	lcount = len(selList)
	printDebug('After get sel meshes')

	# Process each selected item.
	for i in range(0, lcount):
		printDebug('Item #%s' %(i))
		itemID = selList[i]
		printDebug('Processing item "%s"' %(itemID))

		# Process only mesh.
		itemType = lx.eval('query sceneservice item.type ? %s' %(itemID))
		if (itemType == 'mesh'):
			# Gets the GoZBrushID of the mesh.
			lx.eval('select.drop item')
			lx.eval('select.item %s set' %(itemID))
			abort = 0
			#name = lx.eval('item.name ?')
			name = safeGetItemName()
			gozID = lx.eval('item.tag string GOZB ?')

			printDebug('  item ID[%s] has name "%s" and GoZID "%s"' %(itemID, name, gozID))
			# If the GoZBrushID is not same as the mesh name, ignore it and creates an unique GoZBrushID.
			if (gozID != name):
				printDebug('  creating GoZBrushID...')
				gozID = createGoZBrushID(name)
				printDebug('  -> GoZBrushID created is "%s"' %(gozID))

				# Sets the GoZID to this mesh.
				if (gozID == name):
					lx.eval('item.tag string GOZB "%s"' %(gozID))

				# If it already exists another GoZObject having the same name as the mesh,
				# asks the user what to do(replace existing GoZObject, rename the mesh to an unique GoZObject, cancel export).
				else:
					title = "Exporting GoZ object " + name
					msg = "There is already an existing GoZ file [" + name + "] in the GoZ project folder.\n"
					msg = msg + "Do you want to replace it?\n'Yes' to replace, 'No' to rename the mesh " + name + ", 'Cancel' to cancel export"
					res = dialog("GoZ note", msg, "yesNoCancel")
					if (res == "ok"):
						printDebug('  -> itemID=(%s): set GOZB tag to "%s"' %(itemID, name))
						gozID = name
						lx.eval('select.drop item')
						lx.eval('select.item %s set' %(itemID))
						lx.eval('item.tag string GOZB "%s"' %(gozID))
					if (res == "no"):
						name = gozID
						lx.eval('select.drop item')
						lx.eval('select.item %s set' %(itemID))
						lx.eval('item.name "%s"' %(gozID))
						lx.eval('item.tag string GOZB "%s"' %(gozID))
					if (res == "cancel"):
						abort = 1

			# If export was not cancelled by the user, performs the export:
			if (abort == 0):
				printDebug('  exporting the mesh...')
				#Opens the GoZObject list file if needed
				if (exportedCount == 0):
					gozListFile = open(gGoZBrushPath + "GoZ_ObjectList.txt", "w")
					gozListFile.close()

				path = projectPath + name
				printDebug('  exporting the mesh(ID[%s], name "%s", GoZID "%s") to path "%s"' %(itemID, name, gozID, path))
				gozListFile = open(gGoZBrushPath + "GoZ_ObjectList.txt", "a")
				line = path + "\n"
				gozListFile.write(line)
				gozListFile.close()
				exportedCount = exportedCount + 1

				# Create a temporary scene and copy the GoZ object in that temporary scene.
				lx.eval('select.drop item')
				lx.eval('select.item %s' %(itemID))
				lx.eval('select.copy')
				lx.eval('scene.new')
				lx.eval('select.paste')

				# Then saves this temporary scene in a GoZ file.
				path = path + ".GoZ"
				lx.out('GoZ Note: Exporting "%s"' %(path))
				lx.eval('scene.saveAs "%s" io_GoZ true' %(path))

				# And finally destroys this temporary scene.
				lx.eval('!scene.close')


	# Closes the GoZObject list file and launch ZBrush.
	if (exportedCount > 0):
		fileName = gGoZBrushPath + "GoZ_Application.txt"
		exePath = gGoZBrushPath + "GoZBrushFromApp" + gGoZAppExt
		file = open(fileName, "w")
		file.write("Modo")
		file.close()
		launchApp(exePath)
		lx.out('%d meshes exported to ZBrush!!!' %(exportedCount))
	else:
		lx.out('No mesh selected: did not export!!!')

doGoZExport()

