#define _USE_MATH_DEFINES
#include <math.h>
#include <PXFloatServer.h>
#include <memory.h>


  PXVector3f::PXVector3f()
  { m_coord[0]=0.0f, m_coord[1]=0.0f, m_coord[2]=0.0f; }

  PXVector3f::PXVector3f(float x, float y, float z)
  { m_coord[0]=x, m_coord[1]=y, m_coord[2]=z; }

  bool PXVector3f::operator == (const PXVector3f &other) const
  { return (m_coord[0]==other.m_coord[0]) && (m_coord[1]==other.m_coord[1]) && (m_coord[2]==other.m_coord[2]); }

  bool PXVector3f::operator != (const PXVector3f &other) const 
  { return (m_coord[0]!=other.m_coord[0]) || (m_coord[1]!=other.m_coord[1]) || (m_coord[2]!=other.m_coord[2]); }

inline bool
PXVector3f::operator < (const PXVector3f &other) const
{
#if     1
  if (m_coord[0] < other.m_coord[0])
    return true;
  else if (m_coord[0] > other.m_coord[0])
    return false;
  if (m_coord[1] < other.m_coord[1])
    return true;
  else if (m_coord[1] > other.m_coord[1])
    return false;
  if (m_coord[2] < other.m_coord[2])
    return true;
/*
  else if (m_coord[2] > other.m_coord[2])
    return false;
*/
  return false;
#else   // 0
  if (PXFloatServer::isLower(m_coord[0], other.m_coord[0]))
    return true;
  else if (PXFloatServer::isGreater(m_coord[0], other.m_coord[0]))
    return false;
  if (PXFloatServer::isLower(m_coord[1], other.m_coord[1]))
    return true;
  else if (PXFloatServer::isGreater(m_coord[1], other.m_coord[1]))
    return false;
  if (PXFloatServer::isLower(m_coord[2], other.m_coord[2]))
    return true;
/*
  else if (PXFloatServer::isGreater(m_coord[2], other.m_coord[2]))
    return false;
*/
  return false;
#endif  // 0
}



float PXVector3f::x() const   
{ return m_coord[0]; }

float & PXVector3f::x()     
{ return m_coord[0]; }

float PXVector3f::y() const  
{ return m_coord[1]; }

float & PXVector3f::y()    
{ return m_coord[1]; }

float PXVector3f::z() const   
{ return m_coord[2]; }

float & PXVector3f::z()      
{ return m_coord[2]; }

const float * PXVector3f::getValue() const  
{ return m_coord; }

float * PXVector3f::getValue() 
{ return m_coord; }

void PXVector3f::setValue(float x, float y, float z)
{ m_coord[0]=x, m_coord[1]=y, m_coord[2]=z; }

void PXVector3f::setZero() 
{ m_coord[0]=0.0f, m_coord[1]=0.0f, m_coord[2]=0.0f; }

void PXVector3f::swap(PXVector3f &other)
{ float tmp=m_coord[0]; m_coord[0]=other.m_coord[0]; other.m_coord[0]=tmp; tmp=m_coord[1]; m_coord[1]=other.m_coord[1]; other.m_coord[1]=tmp; tmp=m_coord[2]; m_coord[2]=other.m_coord[2]; other.m_coord[2]=tmp; }

PXVector3f PXVector3f::operator + (const PXVector3f &vec) const
{ return PXVector3f(m_coord[0]+vec.m_coord[0], m_coord[1]+vec.m_coord[1], m_coord[2]+vec.m_coord[2]); }

PXVector3f & PXVector3f::operator += (const PXVector3f &vec)   
{ m_coord[0]+=vec.m_coord[0], m_coord[1]+=vec.m_coord[1], m_coord[2]+=vec.m_coord[2]; return *this; }

PXVector3f PXVector3f::operator - (const PXVector3f &vec) const 
{ return PXVector3f(m_coord[0]-vec.m_coord[0], m_coord[1]-vec.m_coord[1], m_coord[2]-vec.m_coord[2]); }

PXVector3f & PXVector3f::operator -= (const PXVector3f &vec)  
{ m_coord[0]-=vec.m_coord[0], m_coord[1]-=vec.m_coord[1], m_coord[2]-=vec.m_coord[2]; return *this; }

// inline friend!
PXVector3f operator - (const PXVector3f &vec)    
{ return PXVector3f(-vec.m_coord[0], -vec.m_coord[1], -vec.m_coord[2]); }

PXVector3f PXVector3f::operator * (float coef) const  
{ return PXVector3f(m_coord[0]*coef, m_coord[1]*coef, m_coord[2]*coef); }

PXVector3f & PXVector3f::operator *= (float coef) 
{ m_coord[0]*=coef, m_coord[1]*=coef, m_coord[2]*=coef; return *this; }

PXVector3f PXVector3f::operator / (float coef) const 
{ return PXVector3f(m_coord[0]/coef, m_coord[1]/coef, m_coord[2]/coef); }

PXVector3f & PXVector3f::operator /= (float coef)
{ m_coord[0]/=coef, m_coord[1]/=coef, m_coord[2]/=coef; return *this; }


PXVector3f PXVector3f::cross(const PXVector3f &vec) const 
{
  return PXVector3f((m_coord[1]*vec.m_coord[2])-(m_coord[2]*vec.m_coord[1]), (m_coord[2]*vec.m_coord[0])-(m_coord[0]*vec.m_coord[2]), (m_coord[0]*vec.m_coord[1])-(m_coord[1]*vec.m_coord[0])); 
}

float PXVector3f::dot(const PXVector3f &vec) const 
{
  return (m_coord[0]*vec.m_coord[0] + m_coord[1]*vec.m_coord[1] + m_coord[2]*vec.m_coord[2]); 
}

float PXVector3f::norm() const 
{
  return sqrtf(m_coord[0]*m_coord[0] + m_coord[1]*m_coord[1] + m_coord[2]*m_coord[2]); 
}

float PXVector3f::norm2() const
{
  return (m_coord[0]*m_coord[0] + m_coord[1]*m_coord[1] + m_coord[2]*m_coord[2]); 
}

float PXVector3f::dist(const PXVector3f &vec) const 
{
  return (*this-vec).norm(); 
}

float PXVector3f::dist2(const PXVector3f &vec) const
{
  return (*this-vec).norm2(); 
}


PXVector3f::PXVector3f(const PXVector3f &pt1, const PXVector3f &pt2)
{
  m_coord[0] = pt2.x() - pt1.x();
  m_coord[1] = pt2.y() - pt1.y();
  m_coord[2] = pt2.z() - pt1.z();
}

void 
PXVector3f::setValue(const float *coords)
{
  memcpy(m_coord, coords, 3 * sizeof(float));
}

void
PXVector3f::copyValuesTo(float *dest) const
{
  memcpy(dest, m_coord, 3 * sizeof(float));
}

double PXVector3f::dotD(const PXVector3f &vec) const                                   
{
  return (m_coord[0]*vec.m_coord[0] + m_coord[1]*vec.m_coord[1] + m_coord[2]*vec.m_coord[2]); 
}

double PXVector3f::normD() const
{
  return sqrt(m_coord[0]*m_coord[0] + m_coord[1]*m_coord[1] + m_coord[2]*m_coord[2]); 
}

double PXVector3f::norm2D() const
{
  return (m_coord[0]*m_coord[0] + m_coord[1]*m_coord[1] + m_coord[2]*m_coord[2]); 
}

double PXVector3f::distD(const PXVector3f &vec) const
{
  return (*this-vec).normD(); 
}

double PXVector3f::dist2D(const PXVector3f &vec) const
{
  return (*this-vec).norm2D(); 
}

bool 
PXVector3f::isEqual(const PXVector3f &other) const               
{
  return (m_coord[0] == other.m_coord[0] && m_coord[1] == m_coord[1] && m_coord[2] == m_coord[2]); 
}

bool 
PXVector3f::isEqualEps(const PXVector3f &other, float eps) const               
 {
   return PXFloatServer::isEqualEps(m_coord[0], other.m_coord[0], eps) && PXFloatServer::isEqualEps(m_coord[1], other.m_coord[1], eps) && PXFloatServer::isEqualEps(m_coord[2], other.m_coord[2], eps); 
}

bool 
PXVector3f::isZero() const
{
  return ((*this) == m_zero); 
}

bool 
PXVector3f::isZeroEps(float eps) const
{
  return PXFloatServer::isZeroEps(m_coord[0], eps) && PXFloatServer::isZeroEps(m_coord[1], eps) && PXFloatServer::isZeroEps(m_coord[2], eps); 
}


//static 
PXVector3f 
PXVector3f::middlePoint(const PXVector3f &P1, const PXVector3f &P2)
{
  return (P1+P2)*0.5f;
}

PXVector3f 
PXVector3f::middlePoint(const PXVector3f &otherP) const
{
  return ((*this)+otherP)*0.5f;
}



PXVector3f operator * (float coef, const PXVector3f &vec)
{
  return PXVector3f(vec.x()*coef, vec.y()*coef, vec.z()*coef); 
}

const float & PXVector3f::operator [](int i) const 
{
  return m_coord[i]; 
}

float & PXVector3f::operator [](int i)
{  return m_coord[i]; }



double
PXVector3f::angleD(const PXVector3f &vec) const
{
  double n = normD() * vec.normD();
  if (n == 0.0)
    return 0.0;
  double a = dotD(vec) / n;
  if (a<=-1.0)
    return M_PI;
  if (a>=1.0)
    return 0.0;

  return acos(a);
}
float
PXVector3f::angleLine(const PXVector3f &vec) const
{
  return (float)angleLineD(vec);
}

double
PXVector3f::angleLineD(const PXVector3f &vec) const
{
  double n = normD() * vec.normD();
  if (n == 0.0)
    return 0.0;
  double a = dotD(vec) / n;
  if (a < 0.0)
    a = -a;
  if (a >= 1.0)
    return 0.0;

  return acos(a);
}

float
PXVector3f::angle(const PXVector3f &vec) const
{
  return (float)angleD(vec);
}

double
PXVector3f::cosAngle(const PXVector3f &vec) const
{	
	double n = normD() * vec.normD();
	if (n != 0.0)
	{
		double s = dotD(vec) / n;
		if (s > 1.0) 
			return 1.0;
		if (s < -1.0) 
			return -1.0;

		return s;
	}
	return 1.0;
}

double
PXVector3f::angleOrientD(const PXVector3f &vec, const PXVector3f &axis) const
{
  PXVector3f cp = cross(vec);
  double angl = angleD(vec);
  if (axis.dot(cp) > 0)
    return angl;
  //else
  if (angl == 0.0)
    return 0.0;
  return 2.0*M_PI - angl;
}
float
PXVector3f::angleOrient(const PXVector3f &vec, const PXVector3f &axis) const
{
  return (float)angleOrientD(vec, axis);
}

// same with normalized vectors
double
PXVector3f::angleDN(const PXVector3f &vec) const
{
  double a = dotD(vec);
  if (a<=-1.0)
    return M_PI;
  if (a>=1.0)
    return 0.0;

  return acos(a);
}
float
PXVector3f::angleN(const PXVector3f &vec) const
{
  return (float)angleDN(vec);
}

double
PXVector3f::angleOrientDN(const PXVector3f &vec, const PXVector3f &axis) const
{
  PXVector3f cp = cross(vec);
  double angl = angleDN(vec);
  if (axis.dot(cp) > 0)
    return angl;
  //else
  if (angl == 0.0)
    return 0.0;
  return 2.0*M_PI - angl;
}

float
PXVector3f::angleOrientN(const PXVector3f &vec, const PXVector3f &axis) const
{
  return (float)angleOrientDN(vec, axis);
}

float
PXVector3f::angleLineN(const PXVector3f &vec) const
{
  return (float)angleLineDN(vec);
}

double
PXVector3f::angleLineDN(const PXVector3f &vec) const
{
  double a = dotD(vec);
  if (a < 0.0)
    a = -a;
  if (a >= 1.0)
    return 0.0;

  return acos(a);
}

double
PXVector3f::cosAngleN(const PXVector3f &vec) const
{	
  double s = dotD(vec);
  if (s > 1.0) 
    return 1.0;
  if (s < -1.0) 
    return -1.0;

  return s;
}





PXVector3f
PXVector3f::multCoords(const PXVector3f &V2) const
{
  return PXVector3f(m_coord[0]*V2.m_coord[0], m_coord[1]*V2.m_coord[1], m_coord[2]*V2.m_coord[2]);
}

void
PXVector3f::selfMultCoords(const PXVector3f &V2)
{
  m_coord[0]*=V2.m_coord[0], m_coord[1]*=V2.m_coord[1], m_coord[2]*=V2.m_coord[2];
}

PXVector3f
PXVector3f::divCoords(const PXVector3f &V2) const
{
  return PXVector3f(m_coord[0]/V2.m_coord[0], m_coord[1]/V2.m_coord[1], m_coord[2]/V2.m_coord[2]);
}

void
PXVector3f::selfDivCoords(const PXVector3f &V2)
{
  m_coord[0]/=V2.m_coord[0], m_coord[1]/=V2.m_coord[1], m_coord[2]/=V2.m_coord[2];
}

// static :
void
PXVector3f::add(PXVector3f &dest, const PXVector3f &vec1, const PXVector3f &vec2)
{
  dest.x() = vec1.x() + vec2.x();
  dest.y() = vec1.y() + vec2.y();
  dest.z() = vec1.z() + vec2.z();
}

// static :
void
PXVector3f::sub(PXVector3f &dest, const PXVector3f &vec1, const PXVector3f &vec2)
{
  dest.x() = vec1.x() - vec2.x();
  dest.y() = vec1.y() - vec2.y();
  dest.z() = vec1.z() - vec2.z();
}

// static :
void
PXVector3f::mult(PXVector3f &dest, const PXVector3f &vec, float factor)
{
  dest.x() = vec.x() * factor;
  dest.y() = vec.y() * factor;
  dest.z() = vec.z() * factor;
}

// static :
void
PXVector3f::inverse(PXVector3f &dest, const PXVector3f &vec)
{
  dest.x() = -vec.x();
  dest.y() = -vec.y();
  dest.z() = -vec.z();
}

// static :
float
PXVector3f::dot(const PXVector3f &vec1, const PXVector3f &vec2)
{
  return vec1.x() * vec2.x() + vec1.y() * vec2.y() + vec1.z() * vec2.z();
}

// static :
double
PXVector3f::dotD(const PXVector3f &vec1, const PXVector3f &vec2)
{
  return (double)vec1.x() * (double)vec2.x()
    + (double)vec1.y() * (double)vec2.y()
    + (double)vec1.z() * (double)vec2.z();
}

// static :
void
PXVector3f::div(PXVector3f &dest, const PXVector3f &vec, float factor)
{
  if (factor==0) return;
  float inv = 1.0f / factor;
  mult(dest, vec, inv); 
}

// static :
void
PXVector3f::cross(PXVector3f &dest, const PXVector3f &vec1, const PXVector3f &vec2)
{
  dest.x() = vec1.y()*vec2.z() - vec1.z()*vec2.y();
  dest.y() = vec1.z()*vec2.x() - vec1.x()*vec2.z();
  dest.z() = vec1.x()*vec2.y() - vec1.y()*vec2.x();
}

inline int
PXVector3f::compareEps(const PXVector3f &other, float eps) const
{
  if (PXFloatServer::isLower(m_coord[0], other.m_coord[0], eps))
    return -1;
  else if (PXFloatServer::isGreater(m_coord[0], other.m_coord[0], eps))
    return 1;
  if (PXFloatServer::isLower(m_coord[1], other.m_coord[1], eps))
    return -1;
  else if (PXFloatServer::isGreater(m_coord[1], other.m_coord[1], eps))
    return 1;
  if (PXFloatServer::isLower(m_coord[2], other.m_coord[2], eps))
    return -1;
  else if (PXFloatServer::isGreater(m_coord[2], other.m_coord[2], eps))
    return 1;
  return 0;
}

inline float
PXVector3f::distance2To(const PXVector3f &vec) const
{
  float dx=vec.m_coord[0]-m_coord[0], dy=vec.m_coord[1]-m_coord[1], dz=vec.m_coord[2]-m_coord[2];
  return ((dx*dx)+(dy*dy)+(dz*dz));
}

inline float
PXVector3f::distanceTo(const PXVector3f &vec) const
{
  float dx=vec.m_coord[0]-m_coord[0], dy=vec.m_coord[1]-m_coord[1], dz=vec.m_coord[2]-m_coord[2];
  return sqrt((dx*dx)+(dy*dy)+(dz*dz));
}

#include <PXVector3d.h>
//explicit inline 
PXVector3f::PXVector3f(const PXVector3d &vec)
{
  m_coord[0] = (float)vec.x();
  m_coord[1] = (float)vec.y();
  m_coord[2] = (float)vec.z();
}

